﻿using Hotline.FlowEngine.Definitions;
using Hotline.FlowEngine.Workflows;
using Hotline.Identity.Roles;
using Hotline.Repository.SqlSugar.Extensions;
using Hotline.Settings;
using Hotline.Share.Dtos;
using Hotline.Share.Dtos.FlowEngine;
using Hotline.Share.Enums.FlowEngine;
using Hotline.Share.Requests;
using Hotline.Users;
using MapsterMapper;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using SqlSugar;
using XF.Domain.Authentications;
using XF.Domain.Exceptions;
using XF.Utility.EnumExtensions;

namespace Hotline.Api.Controllers;

/// <summary>
/// 工作流管理
/// </summary>
public class WorkflowController : BaseController
{
    private readonly IDefinitionDomainService _definitionDomainService;
    private readonly IDefinitionRepository _definitionRepository;
    private readonly IWorkflowDomainService _workflowDomainService;
    private readonly IWorkflowRepository _workflowRepository;
    private readonly IUserRepository _userRepository;
    private readonly ISystemOrganizeRepository _organizeRepository;
    private readonly IRoleRepository _roleRepository;
    private readonly ISessionContext _sessionContext;
    private readonly IMapper _mapper;

    public WorkflowController(
        IDefinitionDomainService definitionDomainService,
        IDefinitionRepository definitionRepository,
        IWorkflowDomainService workflowDomainService,
        IWorkflowRepository workflowRepository,
        IUserRepository userRepository,
        ISystemOrganizeRepository organizeRepository,
        IRoleRepository roleRepository,
        ISessionContext sessionContext,
        IMapper mapper)
    {
        _definitionDomainService = definitionDomainService;
        _definitionRepository = definitionRepository;
        _workflowDomainService = workflowDomainService;
        _workflowRepository = workflowRepository;
        _userRepository = userRepository;
        _organizeRepository = organizeRepository;
        _roleRepository = roleRepository;
        _sessionContext = sessionContext;
        _mapper = mapper;
    }

    /// <summary>
    /// 分页查询流程模板
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    [HttpGet("definition")]
    public async Task<PagedDto<DefinitionDto>> QueryDefinitions([FromQuery] PagedRequest dto)
    {
        //todo 数据量大需重构
        var query1 = await _definitionRepository.Queryable()
            .Where(d => d.Status == EDefinitionStatus.Temporary)
            .ToListAsync();
        var query2 = await _definitionRepository.Queryable()
            .Select(d => new { i = SqlFunc.RowNumber($"{d.Version} desc", d.Code), d })
            .MergeTable()
            .Where(d => d.i == 1)
            .ToListAsync();

        var query = query1.Union(query2.Select(d => d.d));
        var total = query.Count();
        var items = query
            .OrderBy(d => d.Status)
            .ThenByDescending(d => d.CreationTime)
            .Skip(dto.Skip())
            .Take(dto.PageSize)
            .ToList();

        return new PagedDto<DefinitionDto>(total, _mapper.Map<IReadOnlyList<DefinitionDto>>(items));
    }

    /// <summary>
    /// 新增流程模板草稿
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    [HttpPost("definition")]
    public async Task<string> AddDefinition([FromBody] AddDefinitionDto dto)
    {
        return await _definitionDomainService.AddAsync(dto, HttpContext.RequestAborted);
    }

    /// <summary>
    /// 更新流程模板草稿
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    [HttpPut("definition")]
    public async Task UpdateDefinition([FromBody] UpdateDefinitionDto dto)
    {
        var definition = await _definitionRepository.GetAsync(dto.Id, HttpContext.RequestAborted);
        if (definition == null)
            throw UserFriendlyException.SameMessage("无效模板编号");
        if (definition.Status == EDefinitionStatus.Temporary)
        {
            _mapper.Map(dto, definition);
            await _definitionRepository.UpdateAsync(definition, HttpContext.RequestAborted);
        }
        else
        {
            var newDefinition = _mapper.Map<Definition>(dto);
            await _definitionRepository.AddAsync(newDefinition, HttpContext.RequestAborted);
        }
    }

    /// <summary>
    /// 删除草稿
    /// </summary>
    /// <param name="id"></param>
    /// <returns></returns>
    /// <exception cref="UserFriendlyException"></exception>
    [HttpDelete("definition/{id}")]
    public async Task RemoveDefinition(string id)
    {
        var definition = await _definitionRepository.GetAsync(id, HttpContext.RequestAborted);
        if (definition == null) return;
        if (definition.Status != EDefinitionStatus.Temporary)
            throw new UserFriendlyException("已发布模板不能删除");
        await _definitionRepository.RemoveAsync(id, false, HttpContext.RequestAborted);
    }

    /// <summary>
    /// 发布（列表操作）
    /// </summary>
    /// <returns></returns>
    [HttpPost("definition/{id}/publish")]
    public async Task Publish(string id)
    {
        await _definitionDomainService.PublishAsync(id, HttpContext.RequestAborted);
    }

    /// <summary>
    /// 发布（保存并发布）
    /// </summary>
    /// <returns></returns>
    [HttpPost("definition/publish")]
    public async Task Publish([FromBody] AddDefinitionDto dto)
    {
        await _definitionDomainService.PublishAsync(dto, HttpContext.RequestAborted);
    }

    /// <summary>
    /// 启用流程模板
    /// </summary>
    /// <returns></returns>
    /// <exception cref="UserFriendlyException"></exception>
    [HttpPost("definition/enable")]
    public async Task Enable(string id)
    {
        var definition = await _definitionRepository.GetAsync(id, HttpContext.RequestAborted);
        if (definition == null)
            throw UserFriendlyException.SameMessage("无效模板编号");
        if (definition.Status == EDefinitionStatus.Temporary)
            throw UserFriendlyException.SameMessage("草稿需发布才能启用");
        if (definition.Status == EDefinitionStatus.Disable)
        {
            definition.Status = EDefinitionStatus.Enable;
            await _definitionRepository.UpdateAsync(definition, HttpContext.RequestAborted);
        }
    }

    /// <summary>
    /// 禁用流程模板
    /// </summary>
    /// <returns></returns>
    /// <exception cref="UserFriendlyException"></exception>
    [HttpPost("definition/disable")]
    public async Task Disable(string id)
    {
        var definition = await _definitionRepository.GetAsync(id, HttpContext.RequestAborted);
        if (definition == null)
            throw UserFriendlyException.SameMessage("无效模板编号");
        if (definition.Status == EDefinitionStatus.Temporary)
            throw UserFriendlyException.SameMessage("草稿无法禁用");
        if (definition.Status == EDefinitionStatus.Enable)
        {
            definition.Status = EDefinitionStatus.Disable;
            await _definitionRepository.UpdateAsync(definition, HttpContext.RequestAborted);
        }
    }

    /// <summary>
    /// 分页查询流程
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    [HttpGet]
    public async Task<PagedDto<WorkflowDto>> QueryPaged([FromQuery] QueryWorkflowPagedDto dto)
    {
        var (total, items) = await _workflowRepository.Queryable()
            .WhereIF(!string.IsNullOrEmpty(dto.ModuleId), d => d.ModuleId == dto.ModuleId)
            .WhereIF(!string.IsNullOrEmpty(dto.Keyword), d => d.Id.Contains(dto.Keyword) || d.Title.Contains(dto.Keyword))
            .ToPagedListAsync(dto, HttpContext.RequestAborted);

        return new PagedDto<WorkflowDto>(total, _mapper.Map<IReadOnlyList<WorkflowDto>>(items));
    }

    /// <summary>
    /// 查询当前流程下一节点参数
    /// </summary>
    /// <param name="workflowId"></param>
    /// <returns></returns>
    [HttpGet("nextsteps")]
    public async Task<IReadOnlyList<NextStepOptions>> GetNextStepOptions(string workflowId)
    {
        var workflow = await _workflowDomainService.GetWorkflowAsync(workflowId, true, true, cancellationToken: HttpContext.RequestAborted);
        var nextStepDefines = _workflowDomainService.GetNextStepOptions(workflow, HttpContext.RequestAborted);

        //todo 性能问题
        var items = new List<NextStepOptions>();
        foreach (var nextStepDefine in nextStepDefines)
        {
            var options = new NextStepOptions
            {
                Code = nextStepDefine.Code,
                Name = nextStepDefine.Name,
            };

            switch (nextStepDefine.HandlerType)
            {
                case EHandlerType.AssignUser:
                    var users = await _userRepository.QueryAsync(d => nextStepDefine.HandlerClassifies.Contains(d.Id));
                    options.NextSteps = users.Select(d => new KeyValuePair<string, string>(d.Id, d.Name)).ToList();
                    break;
                case EHandlerType.AssignOrg:
                    var orgs = await _organizeRepository.QueryAsync(d => nextStepDefine.HandlerClassifies.Contains(d.OrgCode));
                    options.NextSteps = orgs.Select(d => new KeyValuePair<string, string>(d.OrgCode, d.OrgName)).ToList();
                    break;
                case EHandlerType.Role:
                    var roles = await _roleRepository.Queryable().Includes(d => d.Accounts, d => d.User)
                        .Where(d => nextStepDefine.HandlerClassifies.Contains(d.Name)).ToListAsync();
                    var users1 = roles.SelectMany(d => d.Accounts).Select(d => d.User);
                    if (nextStepDefine.OnlySelfOrg ?? false)
                        users1 = users1.Where(d => d.OrgCode == _sessionContext.RequiredOrgCode);
                    options.NextSteps = users1.Select(d => new KeyValuePair<string, string>(d.Id, d.Name)).ToList();
                    break;
                case EHandlerType.OrgLevel:
                    //当前操作人所属部门的下级部门并且属于配置orgLevel的部门
                    var levels = nextStepDefine.HandlerClassifies.Select(d => int.Parse(d));
                    var orgs1 = await _organizeRepository.QueryAsync(d =>
                        d.IsEnable && d.OrgCode.StartsWith(_sessionContext.RequiredOrgCode) &&
                        levels.Contains(d.OrgLevel));
                    options.NextSteps = orgs1.Select(d => new KeyValuePair<string, string>(d.OrgCode, d.OrgName)).ToList();
                    break;
                case EHandlerType.OrgType:
                    var types = nextStepDefine.HandlerClassifies.Select(d => Enum.Parse<EOrgType>(d));
                    var org2 = await _organizeRepository.QueryAsync(d =>
                        d.IsEnable && d.OrgCode.StartsWith(_sessionContext.RequiredOrgCode) &&
                        types.Contains(d.OrgType));
                    options.NextSteps = org2.Select(d => new KeyValuePair<string, string>(d.OrgCode, d.OrgName)).ToList();
                    break;

                default:
                    throw new ArgumentOutOfRangeException();
            }

            items.Add(options);
        }

        return items;
    }

    /// <summary>
    /// 跳转至任意节点
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    [HttpPost("jump")]
    public async Task Jump([FromBody] RecallDto dto)
    {
        var workflow = await _workflowDomainService.GetWorkflowAsync(dto.WorkflowId, true, true, cancellationToken: HttpContext.RequestAborted);
        await _workflowDomainService.JumpAsync(workflow, dto, HttpContext.RequestAborted);
    }

    /// <summary>
    /// 终止流程
    /// </summary>
    /// <param name="workflowId"></param>
    /// <returns></returns>
    [HttpPost("terminate/{workflowId}")]
    public async Task Terminate(string workflowId)
    {
        await _workflowDomainService.TerminateAsync(workflowId, HttpContext.RequestAborted);
    }

    /// <summary>
    /// 获取跳转参数
    /// </summary>
    /// <param name="workflowId"></param>
    /// <returns></returns>
    [HttpGet("jump")]
    public async Task<IReadOnlyList<StepDefineDto>> GetJumpOptions(string workflowId)
    {
        var workflow = await _workflowDomainService.GetWorkflowAsync(workflowId, true, true, cancellationToken: HttpContext.RequestAborted);
        return _mapper.Map<IReadOnlyList<StepDefineDto>>(workflow.Definition.Steps);
    }

    /// <summary>
    /// 获取撤回参数
    /// </summary>
    /// <param name="workflowId"></param>
    /// <returns></returns>
    [HttpGet("recall")]
    public async Task<IReadOnlyList<StepDefineDto>> GetRecallOptions(string workflowId)
    {
        var workflow = await _workflowDomainService.GetWorkflowAsync(workflowId, true, true, cancellationToken: HttpContext.RequestAborted);
        return _mapper.Map<IReadOnlyList<StepDefineDto>>(workflow.StepBoxes);
    }

    [HttpGet("handlerclassify")]
    public async Task<List<KeyValuePair<string, string>>> GetHanlderClassifies(EHandlerType handlerType)
    {
        switch (handlerType)
        {
            case EHandlerType.Role:
                var roles = await _roleRepository.QueryAsync();
                return roles.Select(d => new KeyValuePair<string, string>(d.Name, d.DisplayName)).ToList();
            case EHandlerType.OrgLevel:
                //todo 一级部门、二级部门...
                return new();
            case EHandlerType.OrgType:
                //todo
                return new();
            case EHandlerType.AssignOrg:
                var orgs = await _organizeRepository.GetOrgJson();
                return orgs.Select(d => new KeyValuePair<string, string>(d.OrgCode, d.OrgName)).ToList();
            case EHandlerType.AssignUser:
            default:
                throw new ArgumentOutOfRangeException(nameof(handlerType), handlerType, null);
        }
    }

    [HttpGet("base-data")]
    public dynamic BaseData()
    {
        return new
        {
            ModuleOptions = WorkflowModule.Modules.ToList(),
            GenderOptions = EnumExts.GetDescriptions<EHandlerType>(),
            CountersignMode = EnumExts.GetDescriptions<ECountersignMode>().Where(d => d.Key != 1)
        };
    }
}