﻿using Hotline.FlowEngine.Definitions;
using Hotline.FlowEngine.Workflows;
using Hotline.SeedData;
using Hotline.Settings;
using Hotline.Share.Dtos.FlowEngine;
using Hotline.Share.Enums.FlowEngine;
using Hotline.Users;
using SqlSugar;
using XF.Domain.Authentications;
using XF.Domain.Dependency;
using XF.Domain.Exceptions;

namespace Hotline.Application.FlowEngine;

public class WorkflowApplication : IWorkflowApplication, IScopeDependency
{
    private readonly IDefinitionDomainService _definitionDomainService;
    private readonly IWorkflowDomainService _workflowDomainService;
    private readonly IWorkflowRepository _workflowRepository;
    private readonly IUserRepository _userRepository;
    private readonly ISessionContext _sessionContext;

    public WorkflowApplication(
        IDefinitionDomainService definitionDomainService,
        IWorkflowDomainService workflowDomainService,
        IWorkflowRepository workflowRepository,
        IUserRepository userRepository,
        ISessionContext sessionContext)
    {
        _definitionDomainService = definitionDomainService;
        _workflowDomainService = workflowDomainService;
        _workflowRepository = workflowRepository;
        _userRepository = userRepository;
        _sessionContext = sessionContext;
    }

    public async Task<string> StartWorkflowAsync(StartWorkflowDto dto, CancellationToken cancellationToken = default)
    {
        var definition = await _definitionDomainService.GetLastVersionDefinitionAsync(dto.DefinitionCode, cancellationToken);
        if (definition == null)
            throw new UserFriendlyException("无效模板名称");
        if (definition.Status != EDefinitionStatus.Enable)
            throw new UserFriendlyException("该模板不可用");

        var workflow = await _workflowDomainService.CreateWorkflowAsync(definition, dto.Title, cancellationToken);
        await _workflowDomainService.StartAsync(workflow, dto, cancellationToken);
        return workflow.Id;
    }

    /// <summary>
    /// 流转至下一节点（节点办理）
    /// </summary>
    /// <param name="dto"></param>
    /// <param name="cancellationToken"></param>
    /// <returns></returns>
    public async Task NextAsync(NextWorkflowDto dto, CancellationToken cancellationToken)
    {
        var workflow = await _workflowDomainService.GetWorkflowAsync(dto.WorkflowId, true, true, cancellationToken: cancellationToken);
        var nextStepBoxDefine = _workflowDomainService.GetStepBoxDefine(workflow.Definition, dto.NextStepCode);
        var isOutOfCallCenter =
            await CheckIfFlowOutOfCallCenterAsync(nextStepBoxDefine, dto.NextMainHandler, cancellationToken);
        var isStartCountersign = nextStepBoxDefine.IsStartCountersign(dto.Handlers.Count);
        await _workflowDomainService.NextAsync(workflow, dto, nextStepBoxDefine, isOutOfCallCenter, isStartCountersign, cancellationToken);
    }

    #region private

    private async Task<bool> CheckIfFlowOutOfCallCenterAsync(StepDefine nextStepBoxDefine, string mainHandler, CancellationToken CancellationToken)
    {
        //todo current is center & next is not center return true
        var isFromCallCenter = IsOrgFromCallCenter(_sessionContext.RequiredOrgCode);
        if (!isFromCallCenter) return false;

        var isOutCallCenter = await CheckNextStepHandlerIsOutCallCenterAsync(nextStepBoxDefine, mainHandler, CancellationToken);
        return isFromCallCenter && isOutCallCenter;
    }

    private async Task<bool> CheckNextStepHandlerIsOutCallCenterAsync(StepDefine nextStepBoxDefine, string mainHandler, CancellationToken cancellationToken)
    {
        if (nextStepBoxDefine.HandlerType is EHandlerType.AssignUser or EHandlerType.Role)
        {
            //mainHandler is userId
            //todo 临时处理方案（暂时耦合user）
            var handler = await _userRepository.GetAsync(mainHandler, cancellationToken);
            if (handler == null)
                throw new UserFriendlyException($"mainHandler未找到对应User, handler: {mainHandler}");
            if (string.IsNullOrEmpty(handler.OrgCode))
                throw UserFriendlyException.SameMessage($"该用户未配置所属部门, userId: {mainHandler}");
            return !IsOrgFromCallCenter(handler.OrgCode);
        }
        else
        {
            //mainHandler is orgCode
            return !IsOrgFromCallCenter(mainHandler);
        }

    }

    /// <summary>
    /// 是否为呼叫中心部门
    /// </summary>
    /// <returns></returns>
    private bool IsOrgFromCallCenter(string orgCode) => orgCode.StartsWith(OrgSeedData.Code);



    #endregion
}