﻿using Hotline.Share.Enums.Settings;
using System.Linq.Expressions;
using XF.Domain.Authentications;
using XF.Domain.Dependency;
using XF.Domain.Entities;

namespace Hotline.Repository.SqlSugar.DataPermissions;

public class DataPermissionFilterBuilder : IDataPermissionFilterBuilder, IScopeDependency
{
    private readonly ISessionContext _sessionContext;

    public DataPermissionFilterBuilder(ISessionContext sessionContext, IDataPermissionManager dataPermissionManager)
    {
        DataPermissionManager = dataPermissionManager;
        _sessionContext = sessionContext;
    }

    public IDataPermissionManager DataPermissionManager { get; }

    public Expression<Func<TEntity, bool>> Build<TEntity>() where TEntity : class, IEntity<string>, IDataPermission, new()
    {
        var userId = _sessionContext.RequiredUserId;
        var scheme = DataPermissionManager.GetQueryFilter<TEntity>(_sessionContext);
        switch (scheme.QueryFilter)
        {
            case EAuthorityType.Create:
                return d => d.CreatorId == userId;
            case EAuthorityType.Org:
                return d => d.CreatorOrgCode == scheme.OrgCode;
            case EAuthorityType.OrgAndBelow:
                return d => d.CreatorOrgCode.StartsWith(scheme.OrgCode);
            case EAuthorityType.All:
                return d => true;
            default:
                return d => false;
        }
    }

    public Expression<Func<TEntity, bool>> BuildIncludeFlowData<TEntity>() where TEntity : class, IEntity<string>, IDataPermission, IWorkflow, new()
    {
        var userId = _sessionContext.RequiredUserId;
        var roles = _sessionContext.Roles;
        var scheme = DataPermissionManager.GetQueryFilter<TEntity>(_sessionContext);
        var (_, depCode, _, _) = DataPermissionManager.GetDataPermissionOptions();

        switch (scheme.QueryFilter)
        {
            case EAuthorityType.Create:
                return d => d.CreatorId == userId || FlowDataFiltering(d, userId, depCode, roles);
            case EAuthorityType.Org:
                return d => d.CreatorOrgCode == scheme.OrgCode || FlowDataFiltering(d, userId, depCode, roles);
            case EAuthorityType.OrgAndBelow:
                return d => d.CreatorOrgCode.StartsWith(scheme.OrgCode) || FlowDataFiltering(d, userId, depCode, roles);
            case EAuthorityType.All:
                return d => true;
            default:
                return d => FlowDataFiltering(d, userId, depCode, roles);
        }
    }

    private static bool FlowDataFiltering<TEntity>(TEntity entity, string userId, string orgCode, string[] roles) where TEntity : class, IEntity<string>, IDataPermission, IWorkflow, new()
    {
        if (entity.AssignUserIds.Contains(userId)) return true;
        foreach (var assignOrgCode in entity.AssignOrgCodes)
        {
            if (assignOrgCode == orgCode) return true;
            var baseOrg = assignOrgCode.Substring(0, 3);
            if (orgCode.StartsWith(baseOrg) && orgCode.Length < assignOrgCode.Length)
                return true;
        }

        return entity.AssignRoles.Intersect(roles).Any();
    }
}