﻿using Hotline.FlowEngine.Definitions;
using Hotline.Share.Enums.FlowEngine;
using SqlSugar;
using XF.Domain.Repository;

namespace Hotline.FlowEngine.Workflows;

public class Workflow : CreationEntity
{
    public string DefinitionId { get; set; }

    [SugarColumn(IsNullable = true)]
    public string? ModuleId { get; set; }

    /// <summary>
    /// 业务模块名称
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? ModuleName { get; set; }

    [SugarColumn(IsNullable = true)]
    public string? ModuleCode { get; set; }

    public string Title { get; set; }

    /// <summary>
    /// 到期时间
    /// </summary>
    public DateTime ExpiredTime { get; set; }

    public DateTime? CompleteTime { get; set; }

    public EWorkflowStatus Status { get; set; }

    /// <summary>
    /// 当前节点名称（会签状态此字段保存最外层会签办理节点名称）
    /// </summary>
    public string CurrentStepName { get; set; }

    /// <summary>
    /// 到达当前节点时间（stepBox创建时间）
    /// </summary>
    public DateTime CurrentStepTime { get; set; }

    /// <summary>
    /// 当前会签办理节点code，嵌套会签为最外层会签办理节点code（不处于会签状态则无值）
    /// </summary>
    public string? CurrentCountersignStepCode { get; set; }


    [Navigate(NavigateType.OneToOne, nameof(DefinitionId))]
    public Definition Definition { get; set; }

    /// <summary>
    /// 补充信息
    /// </summary>
    [Navigate(NavigateType.OneToMany, nameof(WorkflowSupplement.WorkflowId))]
    public List<WorkflowSupplement> Supplements { get; set; }

    /// <summary>
    /// 主节点，依据流转进度动态生成或删除
    /// </summary>
    [SugarColumn(IsIgnore = true)]
    public List<WorkflowStep> StepBoxes { get; set; }

    [SugarColumn(IsIgnore = true)]
    public List<WorkflowTrace> Traces { get; set; } = new();

    #region Mehod

    /// <summary>
    /// 流程结束
    /// </summary>
    public void Complete()
    {
        Status = EWorkflowStatus.Completed;
        CompleteTime = DateTime.Now;
    }

    public void Terminate()
    {
        Status = EWorkflowStatus.Terminated;
        CompleteTime = DateTime.Now;
    }

    /// <summary>
    /// 流程进行流转
    /// </summary>
    public void FLow(string currentStepName, DateTime currentStepTime, string? currentCountersignCode = null)
    {
        CurrentStepName = currentStepName;
        CurrentStepTime = currentStepTime;
        if (!string.IsNullOrEmpty(currentCountersignCode))
            CurrentCountersignStepCode = currentCountersignCode;
    }

    /// <summary>
    /// 检查会签是否结束
    /// </summary>
    public void CompleteCountersign()
    {
        if (IsInCountersign())
        {
            var countersignStepBox = StepBoxes.First(d => d.Code == CurrentCountersignStepCode);
            var isCountersignOver = countersignStepBox.Steps.All(d =>
                d.Status is EWorkflowStepStatus.Completed &&
                (!d.HasStartCountersign || d.IsCountersignComplete.GetValueOrDefault()));
            if (isCountersignOver) //会签结束
                CurrentCountersignStepCode = null;
        }
    }

    public bool CheckIfCountersignOver()
    {
        var countersignStepBox = StepBoxes.First(d => d.Code == CurrentCountersignStepCode);
        var isCountersignOver = countersignStepBox.Steps.All(d =>
            d.Status is EWorkflowStepStatus.Completed &&
            (!d.HasStartCountersign || d.IsCountersignComplete.GetValueOrDefault()));
        return isCountersignOver;
    }

    public void EndCountersign() => CurrentCountersignStepCode = null;

    /// <summary>
    /// 更新workflow中当前停留节点，时间和会签开始节点code
    /// </summary>
    /// <param name="isStartCountersign"></param>
    /// <param name="stepBox"></param>
    public void SetWorkflowCurrentStepInfo(bool isStartCountersign, WorkflowStep stepBox)
    {
        //1.不在会签中，未发起会签(普通处理) 2.不在会签中，发起会签（保存会签节点），3.会签中，不更新
        if (IsInCountersign()) return;
        if (isStartCountersign)
        {
            FLow(stepBox.Name, stepBox.CreationTime, stepBox.Code);
        }
        else
        {
            FLow(stepBox.Name, stepBox.CreationTime);
        }
    }

    /// <summary>
    /// 流程是否处于会签中
    /// </summary>
    /// <returns></returns>
    public bool IsInCountersign() => !string.IsNullOrEmpty(CurrentCountersignStepCode);

    /// <summary>
    /// 结束流程会签状态
    /// </summary>
    public void CloseCountersignStatus() => CurrentCountersignStepCode = null;

    #endregion
}