﻿using Hotline.Share.Enums.FlowEngine;
using SqlSugar;

namespace Hotline.FlowEngine.Workflows;

public class WorkflowStep : StepBasicEntity
{
    [SugarColumn(ColumnDataType = "varchar(2000)", IsJson = true)]
    public List<NextStep> NextSteps { get; set; }

    /// <summary>
    /// 被指派办理对象（依据不同指派方式可能为：depCode或userId）,该字段subStep才会存在，stepBox不存在
    /// </summary>
    public string HandlerId { get; set; }

    /// <summary>
    /// 前一级节点Id(stepBox此字段为上级stepBoxId,step为上级stepId),汇总节点无此字段（可能有多个上级来源）
    /// </summary>
    public string PreviousId { get; set; }

    /// <summary>
    /// 主办
    /// </summary>
    public bool IsMain { get; set; }

    public EWorkflowStepStatus Status { get; set; } = EWorkflowStepStatus.Assigned;

    /// <summary>
    /// stepBox此字段无效，记录stepBox与其下subSteps关系
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? ParentId { get; set; }

    [SugarColumn(IsIgnore = true)]
    public List<WorkflowStep> Steps { get; set; }

    #region 会签

    /// <summary>
    /// 发起会签节点code（不支持发起会签节点无此字段）
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? CountersignStartCode { get; set; }

    /// <summary>
    /// 会签汇总节点code（不支持发起会签节点无此字段）
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? CountersignEndCode { get; set; }

    /// <summary>
    /// 发起会签生成会签Id（不发起会签节点无此字段）
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? StartCountersignId { get; set; }

    /// <summary>
    /// 上级会签Id，（不支持发起会签节点无此字段）
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? PrevCountersignId { get; set; }

    /// <summary>
    /// 汇总节点保存最顶级会签节点的会签Id
    /// <remarks>
    /// 1.如果当前会签流程前还有上级会签流程，当前汇总节点保存上级会签流程的会签Id
    /// 2.如果从汇总节点直接发起会签，上级会签流程的会签Id应该继续往下传递，直到上级会签流程汇总
    /// </remarks>
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? TopCountersignId { get; set; }

    /// <summary>
    /// 当前节点发起的会签是否已结束（未发起会签的节点无此字段）
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public bool? IsCountersignComplete { get; set; }


    /// <summary>
    /// 是否处于会签流程中
    /// </summary>
    public bool IsInCountersign => !string.IsNullOrEmpty(PrevCountersignId);

    /// <summary>
    /// 是否发起会签
    /// </summary>
    public bool HasStartCountersign => !string.IsNullOrEmpty(StartCountersignId);

    #endregion

    #region Method

    /// <summary>
    /// 接办
    /// </summary>
    /// <param name="userId"></param>
    /// <param name="userName"></param>
    public void Accept(string userId, string userName)
    {
        AcceptUserId = userId;
        AcceptUserName = userName;
        AcceptTime = DateTime.Now;
        Status = EWorkflowStepStatus.Accepted;
    }

    /// <summary>
    /// 节点办理完成
    /// </summary>
    /// <param name="userId"></param>
    /// <param name="userName"></param>
    /// <param name="orgCode"></param>
    /// <param name="orgName"></param>
    public void StepComplete(string userId, string userName, string orgCode, string orgName, string nextStepCode)
    {
        UserId = userId;
        UserName = userName;
        OrgCode = orgCode;
        OrgName = orgName;
        CompleteTime = DateTime.Now;
        Status = EWorkflowStepStatus.Completed;

        NextSteps.First(d => d.Code == nextStepCode).Selected = true;
    }

    /// <summary>
    /// 检查stepBox状态，如果子节点全都办理，则stepBox办理完成
    /// </summary>
    public void CheckStepBoxStatusAndUpdate()
    {
        if (Status is not EWorkflowStepStatus.Completed && Steps.All(d => d.Status is EWorkflowStepStatus.Completed))
        {
            Status = EWorkflowStepStatus.Completed;
            CompleteTime = Steps.Max(d => d.CompleteTime);
        }
    }

    /// <summary>
    /// 会签结束
    /// </summary>
    public void CountersignComplete() => IsCountersignComplete = true;

    /// <summary>
    /// 开启会签
    /// </summary>
    public void StartCountersign() => StartCountersignId = Guid.NewGuid().ToString();

    /// <summary>
    /// step设置为可接办状态
    /// </summary>
    public void SetAssigned() => Status = EWorkflowStepStatus.Assigned;

    #endregion

}