﻿using SqlSugar;
using XF.Domain.Entities;
using XF.Domain.Events;

namespace XF.Domain.Repository;

public abstract class Entity : IEntity<string>, IDomainEvents, IDataPermission, ITable
{
    private List<IAppNotification> _domainEvents = new();

    public string Id { get; set; }

    /// <summary>
    /// 组织Id
    /// </summary>
    [SugarColumn(ColumnDescription = "组织Id", IsNullable = true)]
    public string? CreatorOrgId { get; private set; }

    /// <summary>
    /// 组织编码
    /// </summary>
    [SugarColumn(ColumnDescription = "组织编码", IsNullable = true)]
    public string? CreatorOrgCode { get; private set; }

    /// <summary>
    /// 创建人
    /// </summary>
    [SugarColumn(ColumnDescription = "创建人", IsNullable = true)]
    public string? CreatorId { get; private set; }

    /// <summary>
    /// 赋值部门Id
    /// </summary>
    [SugarColumn(ColumnDescription = "数据权限区域Id", IsNullable = true)]
    public string AreaId { get; private set; }

    public void CreateDataPermission(string orgId, string orgCode, string creatorId, string? areaId)
    {
        CreatorOrgId = orgId;
        CreatorOrgCode = orgCode;
        CreatorId = creatorId;
        AreaId = areaId ?? string.Empty;
    }

    public void AddEvent(IAppNotification notification)
    {
        if (_domainEvents.Contains(notification)) return;
        _domainEvents.Add(notification);
    }

    public void ClearEvents() => _domainEvents.Clear();

    public ICollection<IAppNotification> QueryAllEvents() => _domainEvents;
}

/// <summary>
/// 实体（带有创建时间）
/// </summary>
public abstract class CreationEntity : Entity, IHasCreationTime
{
    /// <summary>
    /// 创建时间
    /// </summary>
    [SugarColumn(ColumnDescription = "创建时间")]
    public DateTime CreationTime { get; set; }
}

/// <summary>
/// 实体（带有软删除）
/// </summary>
public abstract class SoftDeleteEntity : Entity, IHasDeletionTime, ISoftDelete
{
    /// <summary>
    /// 删除时间
    /// </summary>
    [SugarColumn(ColumnDescription = "删除时间")]
    public DateTime? DeletionTime { get; private set; }

    [SugarColumn(ColumnDescription = "是否删除")]
    public bool IsDeleted { get; private set; }

    public void SoftDelete()
    {
        IsDeleted = true;
        DeletionTime = DateTime.Now;
    }
}

/// <summary>
/// 实体（带有创建时间和软删除）
/// </summary>
public abstract class CreationSoftDeleteEntity : CreationEntity, IHasDeletionTime, ISoftDelete
{
    [SugarColumn(ColumnDescription = "是否删除")]
    public bool IsDeleted { get; private set; }

    /// <summary>
    /// 删除时间
    /// </summary>
    [SugarColumn(ColumnDescription = "删除时间")]
    public DateTime? DeletionTime { get; set; }

    public void SoftDelete()
    {
        IsDeleted = true;
        DeletionTime = DateTime.Now;
    }

    public void Recover() => IsDeleted = false;
}

public abstract class CreationModificationEntity : CreationEntity, IHasModificationTime
{
    /// <summary>
    /// 最近更新时间
    /// </summary>
    [SugarColumn(ColumnDescription = "最近更新时间")]
    public DateTime? LastModificationTime { get; set; }

    public void Modified() => LastModificationTime = DateTime.Now;
}

/// <summary>
/// 全状态实体(创建时间、更新时间、软删除)
/// </summary>
public abstract class FullStateEntity : CreationSoftDeleteEntity, IHasModificationTime
{
    /// <summary>
    /// 最近更新时间
    /// </summary>
    [SugarColumn(ColumnDescription = "最近更新时间")]
    public DateTime? LastModificationTime { get; set; }

    public void Modified() => LastModificationTime = DateTime.Now;
}

public abstract class WorkflowEntity : FullStateEntity, IWorkflow
{
    [SugarColumn(IsNullable = true)]
    public string? WorkflowId { get; set; }

    [SugarColumn(ColumnDataType = "varchar(1000)", IsJson = true)]
    public List<string> AssignOrgCodes { get; set; } = new();

    [SugarColumn(ColumnDataType = "varchar(2000)", IsJson = true)]
    public List<string> AssignUserIds { get; set; } = new();

    [SugarColumn(ColumnDataType = "varchar(600)", IsJson = true)]
    public List<string> AssignRoles { get; set; } = new();

    public void Assign(EFlowAssignType type, string handler)
    {
        switch (type)
        {
            case EFlowAssignType.Org:
                if (!AssignOrgCodes.Exists(d => d == handler))
                    AssignOrgCodes.Add(handler);
                break;
            case EFlowAssignType.User:
                if (!AssignUserIds.Exists(d => d == handler))
                    AssignUserIds.Add(handler);
                break;
            case EFlowAssignType.Role:
                if (!AssignRoles.Exists(d => d == handler))
                    AssignRoles.Add(handler);
                break;
            default:
                throw new ArgumentOutOfRangeException(nameof(type), type, null);
        }
    }

    public void Assign(EFlowAssignType type, IEnumerable<string> handlers)
    {
        switch (type)
        {
            case EFlowAssignType.Org:
                AssignOrgCodes.AddRange(handlers);
                AssignOrgCodes = AssignOrgCodes.Distinct().ToList();
                break;
            case EFlowAssignType.User:
                AssignUserIds.AddRange(handlers);
                AssignUserIds = AssignUserIds.Distinct().ToList();
                break;
            case EFlowAssignType.Role:
                AssignRoles.AddRange(handlers);
                AssignRoles = AssignRoles.Distinct().ToList();
                break;
            default:
                throw new ArgumentOutOfRangeException(nameof(type), type, null);
        }
    }
}

public abstract class PositionEntity : FullStateEntity
{
    /// <summary>
    /// 经度
    /// </summary>
    [SugarColumn(ColumnDescription = "经度")]
    public double? Longitude { get; set; }

    /// <summary>
    /// 维度
    /// </summary>
    [SugarColumn(ColumnDescription = "维度")]
    public double? Latitude { get; set; }

    /// <summary>
    /// 行政区划编码
    /// </summary>
    [SugarColumn(ColumnDescription = "行政区划编码", IsNullable = true)]
    public string? AreaCode { get; set; }

    [SugarColumn(IsNullable = true)]
    public string? Province { get; set; }

    [SugarColumn(IsNullable = true)]
    public string? City { get; set; }

    [SugarColumn(IsNullable = true)]
    public string? County { get; set; }

    [SugarColumn(IsNullable = true)]
    public string? Street { get; set; }

    [SugarColumn(IsIgnore = true)]
    public string Address => $"{Province ?? string.Empty}{City ?? string.Empty}{County ?? string.Empty}{Street ?? string.Empty}";
}

public abstract class PositionWorkflowEntity : PositionEntity, IWorkflow
{
    [SugarColumn(IsNullable = true)]
    public string? WorkflowId { get; set; }

    [SugarColumn(ColumnDataType = "varchar(1000)", IsJson = true)]
    public List<string> AssignOrgCodes { get; set; } = new();

    [SugarColumn(ColumnDataType = "varchar(2000)", IsJson = true)]
    public List<string> AssignUserIds { get; set; } = new();

    [SugarColumn(ColumnDataType = "varchar(600)", IsJson = true)]
    public List<string> AssignRoles { get; set; } = new();

    public void Assign(EFlowAssignType type, string handler)
    {
        switch (type)
        {
            case EFlowAssignType.Org:
                if (!AssignOrgCodes.Exists(d => d == handler))
                    AssignOrgCodes.Add(handler);
                break;
            case EFlowAssignType.User:
                if (!AssignUserIds.Exists(d => d == handler))
                    AssignUserIds.Add(handler);
                break;
            case EFlowAssignType.Role:
                if (!AssignRoles.Exists(d => d == handler))
                    AssignRoles.Add(handler);
                break;
            default:
                throw new ArgumentOutOfRangeException(nameof(type), type, null);
        }
    }

    public void Assign(EFlowAssignType type, IEnumerable<string> handlers)
    {
        switch (type)
        {
            case EFlowAssignType.Org:
                AssignOrgCodes.AddRange(handlers);
                AssignOrgCodes = AssignOrgCodes.Distinct().ToList();
                break;
            case EFlowAssignType.User:
                AssignUserIds.AddRange(handlers);
                AssignUserIds = AssignUserIds.Distinct().ToList();
                break;
            case EFlowAssignType.Role:
                AssignRoles.AddRange(handlers);
                AssignRoles = AssignRoles.Distinct().ToList();
                break;
            default:
                throw new ArgumentOutOfRangeException(nameof(type), type, null);
        }
    }
}
